@@ -48,6 +48,7 @@
 #include "cogl-attribute-private.h"
 #include "cogl1-context.h"
 #include "cogl-gpu-info-private.h"
+#include "cogl-config-private.h"
 
 #include <string.h>
 
@@ -62,9 +63,11 @@
 
 #ifdef HAVE_COGL_GL
 extern const CoglTextureDriver _cogl_texture_driver_gl;
+extern const CoglDriverVtable _cogl_driver_gl;
 #endif
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
 extern const CoglTextureDriver _cogl_texture_driver_gles;
+extern const CoglDriverVtable _cogl_driver_gles;
 #endif
 
 static void _cogl_context_free (CoglContext *context);
@@ -157,7 +160,7 @@ cogl_context_new (CoglDisplay *display,
 #endif
 
   /* Allocate context memory */
-  context = g_malloc (sizeof (CoglContext));
+  context = g_malloc0 (sizeof (CoglContext));
 
   /* Convert the context into an object immediately in case any of the
      code below wants to verify that the context pointer is a valid
@@ -213,18 +216,11 @@ cogl_context_new (CoglDisplay *display,
      lot throughout Cogl */
   context->driver = display->renderer->driver;
 
-  winsys = _cogl_context_get_winsys (context);
-  if (!winsys->context_init (context, error))
-    {
-      cogl_object_unref (display);
-      g_free (context);
-      return NULL;
-    }
-
   switch (context->driver)
     {
 #ifdef HAVE_COGL_GL
     case COGL_DRIVER_GL:
+      context->driver_vtable = &_cogl_driver_gl;
       context->texture_driver = &_cogl_texture_driver_gl;
       break;
 #endif
@@ -232,6 +228,7 @@ cogl_context_new (CoglDisplay *display,
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
     case COGL_DRIVER_GLES1:
     case COGL_DRIVER_GLES2:
+      context->driver_vtable = &_cogl_driver_gles;
       context->texture_driver = &_cogl_texture_driver_gles;
       break;
 #endif
@@ -240,6 +237,14 @@ cogl_context_new (CoglDisplay *display,
       g_assert_not_reached ();
     }
 
+  winsys = _cogl_context_get_winsys (context);
+  if (!winsys->context_init (context, error))
+    {
+      cogl_object_unref (display);
+      g_free (context);
+      return NULL;
+    }
+
   context->attribute_name_states_hash =
     g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
   context->attribute_name_index_map = NULL;
@@ -258,6 +263,8 @@ cogl_context_new (CoglDisplay *display,
   /* Initialise the driver specific state */
   _cogl_init_feature_overrides (context);
 
+  _context->sampler_cache = _cogl_sampler_cache_new (_context);
+
   _cogl_pipeline_init_default_pipeline ();
   _cogl_pipeline_init_default_layers ();
   _cogl_pipeline_init_state_hash_functions ();
@@ -303,6 +310,8 @@ cogl_context_new (CoglDisplay *display,
   context->current_draw_buffer_state_flushed = 0;
   context->current_draw_buffer_changes = COGL_FRAMEBUFFER_STATE_ALL;
 
+  g_queue_init (&context->gles2_context_stack);
+
   context->journal_flush_attributes_array =
     g_array_new (TRUE, FALSE, sizeof (CoglAttribute *));
   context->journal_clip_bounds = NULL;
@@ -324,8 +333,6 @@ cogl_context_new (CoglDisplay *display,
   context->max_texture_units = -1;
   context->max_activateable_texture_units = -1;
 
-  context->current_program = COGL_INVALID_HANDLE;
-
   context->current_fragment_program_type = COGL_PIPELINE_PROGRAM_TYPE_FIXED;
   context->current_vertex_program_type = COGL_PIPELINE_PROGRAM_TYPE_FIXED;
   context->current_gl_program = 0;
@@ -367,16 +374,16 @@ cogl_context_new (CoglDisplay *display,
 
   context->in_begin_gl_block = FALSE;
 
-  context->quad_buffer_indices_byte = COGL_INVALID_HANDLE;
-  context->quad_buffer_indices = COGL_INVALID_HANDLE;
+  context->quad_buffer_indices_byte = NULL;
+  context->quad_buffer_indices = NULL;
   context->quad_buffer_indices_len = 0;
 
   context->rectangle_byte_indices = NULL;
   context->rectangle_short_indices = NULL;
   context->rectangle_short_indices_len = 0;
 
-  context->texture_download_pipeline = COGL_INVALID_HANDLE;
-  context->blit_texture_pipeline = COGL_INVALID_HANDLE;
+  context->texture_download_pipeline = NULL;
+  context->blit_texture_pipeline = NULL;
 
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
   if (context->driver != COGL_DRIVER_GLES2)
@@ -389,10 +396,11 @@ cogl_context_new (CoglDisplay *display,
     GE (context, glEnable (GL_ALPHA_TEST));
 #endif
 
-  _context->current_modelview_stack = NULL;
-  _context->current_projection_stack = NULL;
-  _cogl_matrix_stack_init_cache (&_context->builtin_flushed_projection);
-  _cogl_matrix_stack_init_cache (&_context->builtin_flushed_modelview);
+  _context->current_modelview_entry = NULL;
+  _context->current_projection_entry = NULL;
+  _cogl_matrix_entry_identity_init (&_context->identity_entry);
+  _cogl_matrix_entry_cache_init (&_context->builtin_flushed_projection);
+  _cogl_matrix_entry_cache_init (&_context->builtin_flushed_modelview);
 
   default_texture_bitmap =
     cogl_bitmap_new_for_data (_context,
@@ -403,29 +411,26 @@ cogl_context_new (CoglDisplay *display,
 
   /* Create default textures used for fall backs */
   context->default_gl_texture_2d_tex =
-    _cogl_texture_2d_new_from_bitmap (default_texture_bitmap,
-                                      COGL_TEXTURE_NONE,
-                                      /* internal format */
-                                      COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                      NULL);
+    cogl_texture_2d_new_from_bitmap (default_texture_bitmap,
+                                     /* internal format */
+                                     COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                     NULL);
   /* If 3D or rectangle textures aren't supported then these should
      just silently return NULL */
   context->default_gl_texture_3d_tex =
-    _cogl_texture_3d_new_from_bitmap (context,
-                                      default_texture_bitmap,
-                                      1, /* height */
-                                      1, /* depth */
-                                      COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                      NULL);
+    cogl_texture_3d_new_from_bitmap (default_texture_bitmap,
+                                     1, /* height */
+                                     1, /* depth */
+                                     COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                     NULL);
   context->default_gl_texture_rect_tex =
-    _cogl_texture_rectangle_new_from_bitmap (default_texture_bitmap,
-                                             COGL_TEXTURE_NONE,
-                                             COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+    cogl_texture_rectangle_new_from_bitmap (default_texture_bitmap,
+                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                            NULL);
 
   cogl_object_unref (default_texture_bitmap);
 
   cogl_push_source (context->opaque_color_pipeline);
-  _cogl_pipeline_flush_gl_state (context->opaque_color_pipeline, FALSE, 0);
 
   context->atlases = NULL;
   g_hook_list_init (&context->atlas_reorganize_callbacks, sizeof (GHook));
@@ -467,14 +472,16 @@ _cogl_context_free (CoglContext *context)
     cogl_object_unref (context->default_gl_texture_rect_tex);
 
   if (context->opaque_color_pipeline)
-    cogl_handle_unref (context->opaque_color_pipeline);
+    cogl_object_unref (context->opaque_color_pipeline);
   if (context->blended_color_pipeline)
-    cogl_handle_unref (context->blended_color_pipeline);
+    cogl_object_unref (context->blended_color_pipeline);
   if (context->texture_pipeline)
-    cogl_handle_unref (context->texture_pipeline);
+    cogl_object_unref (context->texture_pipeline);
 
   if (context->blit_texture_pipeline)
-    cogl_handle_unref (context->blit_texture_pipeline);
+    cogl_object_unref (context->blit_texture_pipeline);
+
+  g_warn_if_fail (context->gles2_context_stack.length == 0);
 
   if (context->journal_flush_attributes_array)
     g_array_free (context->journal_flush_attributes_array, TRUE);
@@ -485,9 +492,9 @@ _cogl_context_free (CoglContext *context)
     g_array_free (context->polygon_vertices, TRUE);
 
   if (context->quad_buffer_indices_byte)
-    cogl_handle_unref (context->quad_buffer_indices_byte);
+    cogl_object_unref (context->quad_buffer_indices_byte);
   if (context->quad_buffer_indices)
-    cogl_handle_unref (context->quad_buffer_indices);
+    cogl_object_unref (context->quad_buffer_indices);
 
   if (context->rectangle_byte_indices)
     cogl_object_unref (context->rectangle_byte_indices);
@@ -495,14 +502,14 @@ _cogl_context_free (CoglContext *context)
     cogl_object_unref (context->rectangle_short_indices);
 
   if (context->default_pipeline)
-    cogl_handle_unref (context->default_pipeline);
+    cogl_object_unref (context->default_pipeline);
 
   if (context->dummy_layer_dependant)
-    cogl_handle_unref (context->dummy_layer_dependant);
+    cogl_object_unref (context->dummy_layer_dependant);
   if (context->default_layer_n)
-    cogl_handle_unref (context->default_layer_n);
+    cogl_object_unref (context->default_layer_n);
   if (context->default_layer_0)
-    cogl_handle_unref (context->default_layer_0);
+    cogl_object_unref (context->default_layer_0);
 
   if (context->current_clip_stack_valid)
     _cogl_clip_stack_unref (context->current_clip_stack);
@@ -521,15 +528,16 @@ _cogl_context_free (CoglContext *context)
   g_slist_free (context->texture_types);
   g_slist_free (context->buffer_types);
 
-  if (_context->current_modelview_stack)
-    cogl_object_unref (_context->current_modelview_stack);
-  if (_context->current_projection_stack)
-    cogl_object_unref (_context->current_projection_stack);
-  _cogl_matrix_stack_destroy_cache (&context->builtin_flushed_projection);
-  _cogl_matrix_stack_destroy_cache (&context->builtin_flushed_modelview);
+  if (_context->current_modelview_entry)
+    _cogl_matrix_entry_unref (_context->current_modelview_entry);
+  if (_context->current_projection_entry)
+    _cogl_matrix_entry_unref (_context->current_projection_entry);
+  _cogl_matrix_entry_cache_destroy (&context->builtin_flushed_projection);
+  _cogl_matrix_entry_cache_destroy (&context->builtin_flushed_modelview);
 
   cogl_pipeline_cache_free (context->pipeline_cache);
 
+  _cogl_sampler_cache_free (context->sampler_cache);
 
   _cogl_destroy_texture_units ();
 
@@ -584,38 +592,133 @@ cogl_egl_context_get_egl_display (CoglContext *context)
 }
 #endif
 
-gboolean
+CoglBool
 _cogl_context_update_features (CoglContext *context,
                                GError **error)
 {
-#ifdef HAVE_COGL_GL
-  if (context->driver == COGL_DRIVER_GL)
-    return _cogl_gl_update_features (context, error);
-#endif
-
-#if defined(HAVE_COGL_GLES) || defined(HAVE_COGL_GLES2)
-  return _cogl_gles_update_features (context, error);
-#endif
-
-  g_assert_not_reached ();
+  return context->driver_vtable->update_features (context, error);
 }
 
 void
-_cogl_context_set_current_projection (CoglContext *context,
-                                      CoglMatrixStack *stack)
+_cogl_context_set_current_projection_entry (CoglContext *context,
+                                            CoglMatrixEntry *entry)
 {
-  cogl_object_ref (stack);
-  if (context->current_projection_stack)
-    cogl_object_unref (context->current_projection_stack);
-  context->current_projection_stack = stack;
+  _cogl_matrix_entry_ref (entry);
+  if (context->current_projection_entry)
+    _cogl_matrix_entry_unref (context->current_projection_entry);
+  context->current_projection_entry = entry;
 }
 
 void
-_cogl_context_set_current_modelview (CoglContext *context,
-                                     CoglMatrixStack *stack)
+_cogl_context_set_current_modelview_entry (CoglContext *context,
+                                           CoglMatrixEntry *entry)
 {
-  cogl_object_ref (stack);
-  if (context->current_modelview_stack)
-    cogl_object_unref (context->current_modelview_stack);
-  context->current_modelview_stack = stack;
+  _cogl_matrix_entry_ref (entry);
+  if (context->current_modelview_entry)
+    _cogl_matrix_entry_unref (context->current_modelview_entry);
+  context->current_modelview_entry = entry;
+}
+
+const char *
+_cogl_context_get_gl_extensions (CoglContext *context)
+{
+  const char *env_disabled_extensions;
+
+  if ((env_disabled_extensions = g_getenv ("COGL_DISABLE_GL_EXTENSIONS"))
+      || _cogl_config_disable_gl_extensions)
+    {
+      static CoglUserDataKey extensions_key;
+      const char *enabled_extensions;
+      char **split_enabled_extensions;
+      char **split_env_disabled_extensions;
+      char **split_conf_disabled_extensions;
+      char **e;
+      GString *result;
+
+      /* We need to return a const string so we'll attach the results
+       * to the CoglContext to avoid leaking the generated string.
+       * This string is only used rarely so we are using
+       * cogl_object_set_user_data instead of adding an explicit
+       * member to CoglContext to avoid making the struct bigger */
+
+      enabled_extensions =
+        cogl_object_get_user_data (COGL_OBJECT (context), &extensions_key);
+      if (enabled_extensions)
+        return enabled_extensions;
+
+      enabled_extensions = (const char *) context->glGetString (GL_EXTENSIONS);
+
+      split_enabled_extensions = g_strsplit (enabled_extensions,
+                                             " ",
+                                             0 /* no max tokens */);
+
+      if (env_disabled_extensions)
+        split_env_disabled_extensions =
+          g_strsplit (env_disabled_extensions,
+                      ",",
+                      0 /* no max tokens */);
+      else
+        split_env_disabled_extensions = NULL;
+
+      if (_cogl_config_disable_gl_extensions)
+        split_conf_disabled_extensions =
+          g_strsplit (_cogl_config_disable_gl_extensions,
+                      ",",
+                      0 /* no max tokens */);
+      else
+        split_conf_disabled_extensions = NULL;
+
+      result = g_string_new (NULL);
+
+      for (e = split_enabled_extensions; *e; e++)
+        {
+          char **d;
+
+          if (split_env_disabled_extensions)
+            for (d = split_env_disabled_extensions; *d; d++)
+              if (!strcmp (*e, *d))
+                goto disabled;
+          if (split_conf_disabled_extensions)
+            for (d = split_conf_disabled_extensions; *d; d++)
+              if (!strcmp (*e, *d))
+                goto disabled;
+
+          if (result->len > 0)
+            g_string_append_c (result, ' ');
+          g_string_append (result, *e);
+
+        disabled:
+          continue;
+        }
+
+      enabled_extensions = g_string_free (result, FALSE);
+
+      g_strfreev (split_enabled_extensions);
+      if (split_env_disabled_extensions)
+        g_strfreev (split_env_disabled_extensions);
+      if (split_conf_disabled_extensions)
+        g_strfreev (split_conf_disabled_extensions);
+
+      cogl_object_set_user_data (COGL_OBJECT (context),
+                                 &extensions_key,
+                                 (void *) enabled_extensions,
+                                 (CoglUserDataDestroyCallback) g_free);
+
+      return enabled_extensions;
+    }
+  else
+    return (const char *) context->glGetString (GL_EXTENSIONS);
+}
+
+const char *
+_cogl_context_get_gl_version (CoglContext *context)
+{
+  const char *version_override;
+
+  if ((version_override = g_getenv ("COGL_OVERRIDE_GL_VERSION")))
+    return version_override;
+  else if (_cogl_config_override_gl_version)
+    return _cogl_config_override_gl_version;
+  else
+    return (const char *) context->glGetString (GL_VERSION);
 }
