@@ -33,7 +33,7 @@
 #include "cogl-texture-3d.h"
 #include "cogl-texture-driver.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-private.h"
 #include "cogl-pipeline-opengl-private.h"
@@ -99,6 +99,15 @@ _cogl_texture_3d_free (CoglTexture3D *tex_3d)
   _cogl_texture_free (COGL_TEXTURE (tex_3d));
 }
 
+static void
+_cogl_texture_3d_set_auto_mipmap (CoglTexture *tex,
+                                  CoglBool value)
+{
+  CoglTexture3D *tex_3d = COGL_TEXTURE_3D (tex);
+
+  tex_3d->auto_mipmap = value;
+}
+
 static CoglTexture3D *
 _cogl_texture_3d_create_base (CoglContext *ctx,
                               int width,
@@ -131,7 +140,7 @@ _cogl_texture_3d_create_base (CoglContext *ctx,
   return tex_3d;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_3d_can_create (CoglContext *ctx,
                              int width,
                              int height,
@@ -167,13 +176,15 @@ _cogl_texture_3d_can_create (CoglContext *ctx,
       return FALSE;
     }
 
-  ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                           &gl_intformat,
-                                           NULL,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          internal_format,
+                                          &gl_intformat,
+                                          NULL,
+                                          &gl_type);
 
   /* Check that the driver can create a texture with that size */
-  if (!ctx->texture_driver->size_supported_3d (GL_TEXTURE_3D,
+  if (!ctx->texture_driver->size_supported_3d (ctx,
+                                               GL_TEXTURE_3D,
                                                gl_intformat,
                                                gl_type,
                                                width,
@@ -213,41 +224,44 @@ cogl_texture_3d_new_with_size (CoglContext *ctx,
                                     error))
     return NULL;
 
-  internal_format = ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                                             &gl_intformat,
-                                                             &gl_format,
-                                                             &gl_type);
+  internal_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                                            internal_format,
+                                                            &gl_intformat,
+                                                            &gl_format,
+                                                            &gl_type);
 
   tex_3d = _cogl_texture_3d_create_base (ctx,
                                          width, height, depth,
                                          internal_format);
 
-  ctx->texture_driver->gen (GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
+  ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_3D,
                                    tex_3d->gl_texture,
                                    FALSE);
   GE( ctx, glTexImage3D (GL_TEXTURE_3D, 0, gl_intformat,
                          width, height, depth, 0, gl_format, gl_type, NULL) );
 
-  return _cogl_texture_3d_handle_new (tex_3d);
+  return _cogl_texture_3d_object_new (tex_3d);
 }
 
 CoglTexture3D *
-_cogl_texture_3d_new_from_bitmap (CoglContext *ctx,
-                                  CoglBitmap *bmp,
-                                  unsigned int height,
-                                  unsigned int depth,
-                                  CoglPixelFormat internal_format,
-                                  GError **error)
+cogl_texture_3d_new_from_bitmap (CoglBitmap *bmp,
+                                 unsigned int height,
+                                 unsigned int depth,
+                                 CoglPixelFormat internal_format,
+                                 GError **error)
 {
-  CoglTexture3D   *tex_3d;
-  CoglBitmap      *dst_bmp;
-  CoglPixelFormat  bmp_format;
-  unsigned int     bmp_width;
-  GLenum           gl_intformat;
-  GLenum           gl_format;
-  GLenum           gl_type;
-  guint8          *data;
+  CoglTexture3D *tex_3d;
+  CoglBitmap *dst_bmp;
+  CoglPixelFormat bmp_format;
+  unsigned int bmp_width;
+  GLenum gl_intformat;
+  GLenum gl_format;
+  GLenum gl_type;
+  uint8_t *data;
+  CoglContext *ctx;
+
+  ctx = _cogl_bitmap_get_context (bmp);
 
   bmp_width = cogl_bitmap_get_width (bmp);
   bmp_format = cogl_bitmap_get_format (bmp);
@@ -294,9 +308,10 @@ _cogl_texture_3d_new_from_bitmap (CoglContext *ctx,
       _cogl_bitmap_unmap (dst_bmp);
     }
 
-  ctx->texture_driver->gen (GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
+  ctx->texture_driver->gen (ctx, GL_TEXTURE_3D, 1, &tex_3d->gl_texture);
 
-  ctx->texture_driver->upload_to_gl_3d (GL_TEXTURE_3D,
+  ctx->texture_driver->upload_to_gl_3d (ctx,
+                                        GL_TEXTURE_3D,
                                         tex_3d->gl_texture,
                                         FALSE, /* is_foreign */
                                         height,
@@ -310,7 +325,7 @@ _cogl_texture_3d_new_from_bitmap (CoglContext *ctx,
 
   cogl_object_unref (dst_bmp);
 
-  return _cogl_texture_3d_handle_new (tex_3d);
+  return _cogl_texture_3d_object_new (tex_3d);
 }
 
 CoglTexture3D *
@@ -322,7 +337,7 @@ cogl_texture_3d_new_from_data (CoglContext *context,
                                CoglPixelFormat internal_format,
                                int rowstride,
                                int image_stride,
-                               const guint8 *data,
+                               const uint8_t *data,
                                GError **error)
 {
   CoglBitmap *bitmap;
@@ -353,7 +368,7 @@ cogl_texture_3d_new_from_data (CoglContext *context,
      recommends avoiding this situation. */
   if (image_stride % rowstride != 0)
     {
-      guint8 *bmp_data;
+      uint8_t *bmp_data;
       int bmp_rowstride;
       int z, y;
 
@@ -390,14 +405,13 @@ cogl_texture_3d_new_from_data (CoglContext *context,
                                        image_stride / rowstride * depth,
                                        format,
                                        rowstride,
-                                       (guint8 *) data);
+                                       (uint8_t *) data);
 
-  ret = _cogl_texture_3d_new_from_bitmap (context,
-                                          bitmap,
-                                          height,
-                                          depth,
-                                          internal_format,
-                                          error);
+  ret = cogl_texture_3d_new_from_bitmap (bitmap,
+                                         height,
+                                         depth,
+                                         internal_format,
+                                         error);
 
   cogl_object_unref (bitmap);
 
@@ -410,13 +424,13 @@ _cogl_texture_3d_get_max_waste (CoglTexture *tex)
   return -1;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_3d_is_sliced (CoglTexture *tex)
 {
   return FALSE;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_3d_can_hardware_repeat (CoglTexture *tex)
 {
   return TRUE;
@@ -438,7 +452,7 @@ _cogl_texture_3d_transform_quad_coords_to_gl (CoglTexture *tex,
   /* The texture coordinates map directly so we don't need to do
      anything other than check for repeats */
 
-  gboolean need_repeat = FALSE;
+  CoglBool need_repeat = FALSE;
   int i;
 
   for (i = 0; i < 4; i++)
@@ -449,7 +463,7 @@ _cogl_texture_3d_transform_quad_coords_to_gl (CoglTexture *tex,
           : COGL_TRANSFORM_NO_REPEAT);
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_3d_get_gl_texture (CoglTexture *tex,
                                  GLuint *out_gl_handle,
                                  GLenum *out_gl_target)
@@ -508,7 +522,7 @@ _cogl_texture_3d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
          available we'll fallback to temporarily enabling
          GL_GENERATE_MIPMAP and reuploading the first pixel */
       if (cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
-        ctx->texture_driver->gl_generate_mipmaps (GL_TEXTURE_3D);
+        ctx->texture_driver->gl_generate_mipmaps (ctx, GL_TEXTURE_3D);
 #if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
       else if (ctx->driver != COGL_DRIVER_GLES2)
         {
@@ -542,7 +556,7 @@ _cogl_texture_3d_ensure_non_quad_rendering (CoglTexture *tex)
   /* Nothing needs to be done */
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_3d_set_region (CoglTexture    *tex,
                              int             src_x,
                              int             src_y,
@@ -558,10 +572,10 @@ _cogl_texture_3d_set_region (CoglTexture    *tex,
 }
 
 static int
-_cogl_texture_3d_get_data (CoglTexture     *tex,
-                           CoglPixelFormat  format,
-                           unsigned int     rowstride,
-                           guint8          *data)
+_cogl_texture_3d_get_data (CoglTexture *tex,
+                           CoglPixelFormat format,
+                           unsigned int rowstride,
+                           uint8_t *data)
 {
   /* FIXME: we could probably implement this by assuming the data is
      big enough to hold all of the images and that there is no stride
@@ -604,6 +618,7 @@ _cogl_texture_3d_get_type (CoglTexture *tex)
 static const CoglTextureVtable
 cogl_texture_3d_vtable =
   {
+    TRUE, /* primitive */
     _cogl_texture_3d_set_region,
     _cogl_texture_3d_get_data,
     NULL, /* foreach_sub_texture_in_region */
@@ -622,5 +637,6 @@ cogl_texture_3d_vtable =
     _cogl_texture_3d_get_width,
     _cogl_texture_3d_get_height,
     _cogl_texture_3d_get_type,
-    NULL /* is_foreign */
+    NULL, /* is_foreign */
+    _cogl_texture_3d_set_auto_mipmap
   };
