@@ -34,7 +34,7 @@
 #include "cogl-texture-rectangle-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-context-private.h"
-#include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-opengl-private.h"
 
@@ -58,7 +58,7 @@ COGL_TEXTURE_DEFINE (TextureRectangle, texture_rectangle);
 
 static const CoglTextureVtable cogl_texture_rectangle_vtable;
 
-static gboolean
+static CoglBool
 can_use_wrap_mode (GLenum wrap_mode)
 {
   return (wrap_mode == GL_CLAMP ||
@@ -108,13 +108,14 @@ _cogl_texture_rectangle_free (CoglTextureRectangle *tex_rect)
   _cogl_texture_free (COGL_TEXTURE (tex_rect));
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_can_create (unsigned int width,
                                     unsigned int height,
                                     CoglPixelFormat internal_format,
                                     GError **error)
 {
   GLenum gl_intformat;
+  GLenum gl_format;
   GLenum gl_type;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
@@ -128,14 +129,17 @@ _cogl_texture_rectangle_can_create (unsigned int width,
       return FALSE;
     }
 
-  ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                           &gl_intformat,
-                                           NULL,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          internal_format,
+                                          &gl_intformat,
+                                          &gl_format,
+                                          &gl_type);
 
   /* Check that the driver can create a texture with that size */
-  if (!ctx->texture_driver->size_supported (GL_TEXTURE_RECTANGLE_ARB,
+  if (!ctx->texture_driver->size_supported (ctx,
+                                            GL_TEXTURE_RECTANGLE_ARB,
                                             gl_intformat,
+                                            gl_format,
                                             gl_type,
                                             width,
                                             height))
@@ -150,6 +154,14 @@ _cogl_texture_rectangle_can_create (unsigned int width,
   return TRUE;
 }
 
+static void
+_cogl_texture_rectangle_set_auto_mipmap (CoglTexture *tex,
+                                         CoglBool value)
+{
+  /* Rectangle textures currently never support mipmapping so there's
+     no point in doing anything here */
+}
+
 static CoglTextureRectangle *
 _cogl_texture_rectangle_create_base (unsigned int     width,
                                      unsigned int     height,
@@ -196,15 +208,19 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
                                            internal_format, error))
     return NULL;
 
-  internal_format = ctx->texture_driver->pixel_format_to_gl (internal_format,
-                                                             &gl_intformat,
-                                                             &gl_format,
-                                                             &gl_type);
+  internal_format = ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                                            internal_format,
+                                                            &gl_intformat,
+                                                            &gl_format,
+                                                            &gl_type);
 
   tex_rect = _cogl_texture_rectangle_create_base (width, height,
                                                   internal_format);
 
-  ctx->texture_driver->gen (GL_TEXTURE_RECTANGLE_ARB, 1, &tex_rect->gl_texture);
+  ctx->texture_driver->gen (ctx,
+                            GL_TEXTURE_RECTANGLE_ARB,
+                            1, /* num textures */
+                            &tex_rect->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB,
                                    tex_rect->gl_texture,
                                    tex_rect->is_foreign);
@@ -215,20 +231,21 @@ cogl_texture_rectangle_new_with_size (CoglContext *ctx,
 }
 
 CoglTextureRectangle *
-_cogl_texture_rectangle_new_from_bitmap (CoglBitmap      *bmp,
-                                         CoglTextureFlags flags,
-                                         CoglPixelFormat  internal_format)
+cogl_texture_rectangle_new_from_bitmap (CoglBitmap *bmp,
+                                        CoglPixelFormat internal_format,
+                                        GError **error)
 {
   CoglTextureRectangle *tex_rect;
   CoglBitmap           *dst_bmp;
   GLenum                gl_intformat;
   GLenum                gl_format;
   GLenum                gl_type;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
+  CoglContext          *ctx;
 
   _COGL_RETURN_VAL_IF_FAIL (cogl_is_bitmap (bmp), NULL);
 
+  ctx = _cogl_bitmap_get_context (bmp);
+
   internal_format =
     _cogl_texture_determine_internal_format (cogl_bitmap_get_format (bmp),
                                              internal_format);
@@ -236,7 +253,7 @@ _cogl_texture_rectangle_new_from_bitmap (CoglBitmap      *bmp,
   if (!_cogl_texture_rectangle_can_create (cogl_bitmap_get_width (bmp),
                                            cogl_bitmap_get_height (bmp),
                                            internal_format,
-                                           NULL))
+                                           error))
     return NULL;
 
   dst_bmp = _cogl_texture_prepare_for_upload (bmp,
@@ -253,8 +270,12 @@ _cogl_texture_rectangle_new_from_bitmap (CoglBitmap      *bmp,
                                                   cogl_bitmap_get_height (bmp),
                                                   internal_format);
 
-  ctx->texture_driver->gen (GL_TEXTURE_RECTANGLE_ARB, 1, &tex_rect->gl_texture);
-  ctx->texture_driver->upload_to_gl (GL_TEXTURE_RECTANGLE_ARB,
+  ctx->texture_driver->gen (ctx,
+                            GL_TEXTURE_RECTANGLE_ARB,
+                            1, /* num textures */
+                            &tex_rect->gl_texture);
+  ctx->texture_driver->upload_to_gl (ctx,
+                                     GL_TEXTURE_RECTANGLE_ARB,
                                      tex_rect->gl_texture,
                                      FALSE,
                                      dst_bmp,
@@ -286,7 +307,8 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
 
   _COGL_GET_CONTEXT (ctx, NULL);
 
-  if (!ctx->texture_driver->allows_foreign_gl_target (GL_TEXTURE_RECTANGLE_ARB))
+  if (!ctx->texture_driver->allows_foreign_gl_target (ctx,
+                                                      GL_TEXTURE_RECTANGLE_ARB))
     return NULL;
 
   /* Make sure it is a valid GL texture object */
@@ -320,8 +342,9 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
 
       /* If we can query GL for the actual pixel format then we'll ignore
          the passed in format and use that. */
-      if (!ctx->texture_driver->pixel_format_from_gl_internal (gl_int_format,
-                                                               &format))
+      if (!ctx->driver_vtable->pixel_format_from_gl_internal (ctx,
+                                                              gl_int_format,
+                                                              &format))
         return NULL;
     }
   else
@@ -329,10 +352,11 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
     {
       /* Otherwise we'll assume we can derive the GL format from the
          passed in format */
-      ctx->texture_driver->pixel_format_to_gl (format,
-                                               &gl_int_format,
-                                               NULL,
-                                               NULL);
+      ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                              format,
+                                              &gl_int_format,
+                                              NULL,
+                                              NULL);
     }
 
   /* Note: We always trust the given width and height without querying
@@ -365,7 +389,7 @@ _cogl_texture_rectangle_new_from_foreign (GLuint gl_handle,
   tex_rect->min_filter = GL_FALSE;
   tex_rect->mag_filter = GL_FALSE;
 
-  return _cogl_texture_rectangle_handle_new (tex_rect);
+  return _cogl_texture_rectangle_object_new (tex_rect);
 }
 
 static int
@@ -374,13 +398,13 @@ _cogl_texture_rectangle_get_max_waste (CoglTexture *tex)
   return -1;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_is_sliced (CoglTexture *tex)
 {
   return FALSE;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_can_hardware_repeat (CoglTexture *tex)
 {
   return FALSE;
@@ -402,7 +426,7 @@ _cogl_texture_rectangle_transform_quad_coords_to_gl (CoglTexture *tex,
                                                      float *coords)
 {
   CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
-  gboolean need_repeat = FALSE;
+  CoglBool need_repeat = FALSE;
   int i;
 
   for (i = 0; i < 4; i++)
@@ -416,7 +440,7 @@ _cogl_texture_rectangle_transform_quad_coords_to_gl (CoglTexture *tex,
           : COGL_TRANSFORM_NO_REPEAT);
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_get_gl_texture (CoglTexture *tex,
                                         GLuint *out_gl_handle,
                                         GLenum *out_gl_target)
@@ -478,7 +502,7 @@ _cogl_texture_rectangle_ensure_non_quad_rendering (CoglTexture *tex)
   /* Nothing needs to be done */
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_set_region (CoglTexture    *tex,
                                     int             src_x,
                                     int             src_y,
@@ -502,7 +526,8 @@ _cogl_texture_rectangle_set_region (CoglTexture    *tex,
                                           &gl_type);
 
   /* Send data to GL */
-  ctx->texture_driver->upload_subregion_to_gl (GL_TEXTURE_RECTANGLE_ARB,
+  ctx->texture_driver->upload_subregion_to_gl (ctx,
+                                               GL_TEXTURE_RECTANGLE_ARB,
                                                tex_rect->gl_texture,
                                                FALSE,
                                                src_x, src_y,
@@ -517,11 +542,11 @@ _cogl_texture_rectangle_set_region (CoglTexture    *tex,
   return TRUE;
 }
 
-static gboolean
-_cogl_texture_rectangle_get_data (CoglTexture     *tex,
-                                  CoglPixelFormat  format,
-                                  unsigned int     rowstride,
-                                  guint8          *data)
+static CoglBool
+_cogl_texture_rectangle_get_data (CoglTexture *tex,
+                                  CoglPixelFormat format,
+                                  unsigned int rowstride,
+                                  uint8_t *data)
 {
   CoglTextureRectangle *tex_rect = COGL_TEXTURE_RECTANGLE (tex);
   int                   bpp;
@@ -532,17 +557,22 @@ _cogl_texture_rectangle_get_data (CoglTexture     *tex,
 
   bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
 
-  ctx->texture_driver->pixel_format_to_gl (format,
-                                           NULL, /* internal format */
-                                           &gl_format,
-                                           &gl_type);
+  ctx->driver_vtable->pixel_format_to_gl (ctx,
+                                          format,
+                                          NULL, /* internal format */
+                                          &gl_format,
+                                          &gl_type);
 
-  ctx->texture_driver->prep_gl_for_pixels_download (rowstride, bpp);
+  ctx->texture_driver->prep_gl_for_pixels_download (ctx,
+                                                    rowstride,
+                                                    tex_rect->width,
+                                                    bpp);
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_RECTANGLE_ARB,
                                    tex_rect->gl_texture,
                                    tex_rect->is_foreign);
-  return ctx->texture_driver->gl_get_tex_image (GL_TEXTURE_RECTANGLE_ARB,
+  return ctx->texture_driver->gl_get_tex_image (ctx,
+                                                GL_TEXTURE_RECTANGLE_ARB,
                                                 gl_format,
                                                 gl_type,
                                                 data);
@@ -572,7 +602,7 @@ _cogl_texture_rectangle_get_height (CoglTexture *tex)
   return COGL_TEXTURE_RECTANGLE (tex)->height;
 }
 
-static gboolean
+static CoglBool
 _cogl_texture_rectangle_is_foreign (CoglTexture *tex)
 {
   return COGL_TEXTURE_RECTANGLE (tex)->is_foreign;
@@ -587,6 +617,7 @@ _cogl_texture_rectangle_get_type (CoglTexture *tex)
 static const CoglTextureVtable
 cogl_texture_rectangle_vtable =
   {
+    TRUE, /* primitive */
     _cogl_texture_rectangle_set_region,
     _cogl_texture_rectangle_get_data,
     NULL, /* foreach_sub_texture_in_region */
@@ -605,5 +636,6 @@ cogl_texture_rectangle_vtable =
     _cogl_texture_rectangle_get_width,
     _cogl_texture_rectangle_get_height,
     _cogl_texture_rectangle_get_type,
-    _cogl_texture_rectangle_is_foreign
+    _cogl_texture_rectangle_is_foreign,
+    _cogl_texture_rectangle_set_auto_mipmap
   };
