@@ -9,11 +9,12 @@
 
 typedef enum _TestFlags
 {
-  TEST_KNOWN_FAILURE            = 1<<0,
-  TEST_REQUIREMENT_GL           = 1<<1,
-  TEST_REQUIREMENT_NPOT         = 1<<2,
-  TEST_REQUIREMENT_TEXTURE_3D   = 1<<3,
-  TEST_REQUIREMENT_POINT_SPRITE = 1<<4
+  TEST_KNOWN_FAILURE             = 1<<0,
+  TEST_REQUIREMENT_GL            = 1<<1,
+  TEST_REQUIREMENT_NPOT          = 1<<2,
+  TEST_REQUIREMENT_TEXTURE_3D    = 1<<3,
+  TEST_REQUIREMENT_POINT_SPRITE  = 1<<4,
+  TEST_REQUIREMENT_GLES2_CONTEXT = 1<<5
 } TestFlags;
 
 extern CoglContext *ctx;
@@ -41,7 +42,7 @@ test_utils_fini (void);
  */
 void
 test_utils_check_pixel (CoglFramebuffer *framebuffer,
-                        int x, int y, guint32 expected_pixel);
+                        int x, int y, uint32_t expected_pixel);
 
 /*
  * test_utils_check_pixel:
@@ -79,7 +80,7 @@ void
 test_utils_check_region (CoglFramebuffer *framebuffer,
                          int x, int y,
                          int width, int height,
-                         guint32 expected_rgba);
+                         uint32_t expected_rgba);
 
 /*
  * test_utils_compare_pixel:
@@ -92,7 +93,7 @@ test_utils_check_region (CoglFramebuffer *framebuffer,
  * message.
  */
 void
-test_utils_compare_pixel (const guint8 *screen_pixel, guint32 expected_pixel);
+test_utils_compare_pixel (const uint8_t *screen_pixel, uint32_t expected_pixel);
 
 /*
  * test_utils_create_color_texture:
@@ -103,13 +104,13 @@ test_utils_compare_pixel (const guint8 *screen_pixel, guint32 expected_pixel);
  */
 CoglTexture *
 test_utils_create_color_texture (CoglContext *context,
-                                 guint32 color);
+                                 uint32_t color);
 
 /* cogl_test_verbose:
  *
  * Queries if the user asked for verbose output or not.
  */
-gboolean
+CoglBool
 cogl_test_verbose (void);
 
 #endif /* _TEST_UTILS_H_ */
